package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.AuthFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.Token;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {
    @Autowired
    private AuthFeign authFeign;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private RabbitTemplate rabbitTemplate;
    @Override
    public Result authLogin(User user) {
        //非空校验
        if(StringUtils.isBlank( user.getUserName() )){
            return Result.error(500,"请输入用户名");
        }
        //非空校验
        if(StringUtils.isBlank( user.getUserPwd() )){
            return Result.error(500,"请输入密码");
        }
        //获取用户信息
        Result<User> userResult = authFeign.userLogin( user );
        User data = userResult.getData();
        //如果库内不存在该用户
        if(data==null){
            return Result.error(500,"该系统库内未查询到用户信息 请注册");
        }
        //判断密码
        if(!user.getUserPwd().equals( data.getUserPwd() )){
            return Result.error(500,"密码有误");
        }
        //存UUID字符串
        Map<String, Object> map = new HashMap<>();
        String replace = UUID.randomUUID().toString().replace( "-", "" );
        map.put( JwtConstants.USER_KEY,replace );
        //存redis
        redisTemplate.opsForValue().set( "token:"+replace, JSONObject.toJSONString( data ),30, TimeUnit.MINUTES );
        //token令牌
        String token = JwtUtils.createToken( map );
        Token token1 = new Token();
        token1.setToken( token );
        token1.setTime( "三十分钟内生效" );
        //用户每次登陆系统需要给用户发送短息
        rabbitTemplate.convertAndSend( "sendLoginMsg",user.getUserTel(),message -> {
            return message;
        } );
        return Result.success(token1);
    }

    @Override
    public Result addUser(User user) {
        //用户使用手机号注册，注册成功后给用户发送短信
        Result addUserResult = authFeign.addUser( user );
        //模拟失败场景

        //成功
        rabbitTemplate.convertAndSend( "sendAddMsg",user.getUserTel(),message -> {
            return message;
        } );
        return Result.success("注册成功");
    }

    @Override
    public Result info() {
        String token = request.getHeader( "token" );
        String userKey = JwtUtils.getUserKey( token );
        String s = redisTemplate.opsForValue().get( "token:" + userKey );
        User user = JSONObject.parseObject( s, User.class );
        return Result.success(user);
    }
}
