package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Seckill;
import com.bwie.common.result.Result;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsSeckillService;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class EsSeckillServiceImpl implements EsSeckillService {
    @Autowired
    private RestHighLevelClient restHighLevelClient;
    //索引名
    private static final String INDEX_NAME="seckill";
    @Override
    public Result<List<Seckill>> seckillSeach(Seckill seckill) {
        ArrayList<Seckill> tables = new ArrayList<>();
        SearchRequest searchRequest = new SearchRequest(INDEX_NAME);
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        if(StringUtils.isNotBlank( seckill.getSeckillName() )){
            boolQueryBuilder.must(QueryBuilders.matchQuery( "seckillName", seckill.getSeckillName() ));
        }
        if( seckill.getTypeId()!=null){
            boolQueryBuilder.must(QueryBuilders.matchQuery( "typeId", seckill.getTypeId() ));
        }
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder.highlighter(new HighlightBuilder()
                .field( "seckillName" )
                .preTags("<span style=\"color:red\">")
                .postTags("</span>")
        );
        searchRequest.source(searchSourceBuilder);
        try {
            SearchResponse search = restHighLevelClient.search( searchRequest, RequestOptions.DEFAULT );
            SearchHits hits = search.getHits();
            SearchHit[] hits1 = hits.getHits();
            for (SearchHit documentFields : hits1) {
                String sourceAsString = documentFields.getSourceAsString();
                Seckill seckill1 = JSONObject.parseObject( sourceAsString, Seckill.class );
                seckill1.setShopId( Integer.valueOf( documentFields.getId() ) );
                Map<String, HighlightField> highlightFields = documentFields.getHighlightFields();
                if(highlightFields!=null){
                    HighlightField seckillName = highlightFields.get( "seckillName" );
                    if(seckillName!=null){
                        String str= "";
                        Text[] fragments = seckillName.fragments();
                        for (Text fragment : fragments) {
                            str+=fragment;
                        }
                        seckill1.setSeckillName( str );
                    }
                }
                tables.add( seckill1 );
            }
        } catch (IOException e) {
            throw new RuntimeException( e );
        }
        return Result.success(tables);
    }
//      批量添加
    @Override
    public void batchAddSeckill(List<Seckill> seckills) {
        BulkRequest bulkRequest = new BulkRequest();
        seckills.forEach( seckill -> {
            bulkRequest.add( new IndexRequest(INDEX_NAME)
                    .id(  seckill.getSeckillId()+"")
                    .source(JSONObject.toJSONString( seckill ),XContentType.JSON ));
        } );
        try {
            restHighLevelClient.bulk( bulkRequest,RequestOptions.DEFAULT );
        } catch (IOException e) {
            throw new RuntimeException( e );
        }
    }
//    删
    @Override
    public void batchDelSeckill(List<Integer> idsList) {
        BulkRequest bulkRequest = new BulkRequest();
        idsList.forEach( integer -> {
            bulkRequest.add( new DeleteRequest(INDEX_NAME,integer+""));
        } );
        try {
            restHighLevelClient.bulk( bulkRequest,RequestOptions.DEFAULT );
        } catch (IOException e) {
            throw new RuntimeException( e );
        }
    }



}
