package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Shop;
import com.bwie.common.result.Result;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsShopService;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class EsShopServiceImpl implements EsShopService {
    @Autowired
    private RestHighLevelClient restHighLevelClient;
    //索引名
    private static final String INDEX_NAME="n6";
    @Override
    public Result<List<Shop>> shopSeach(Shop shop) {


        ArrayList<Shop> tables = new ArrayList<>();
        //根据索引名创建请求
        SearchRequest searchRequest = new SearchRequest(INDEX_NAME);
        //查询对象
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        //查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        //查询名称
        if(StringUtils.isNotBlank( shop.getShopName() )){
            boolQueryBuilder.must(QueryBuilders.matchQuery( "shopName", shop.getShopName() ));
        }
        //时间区间
        if(shop.getShopTime()!=null){
            boolQueryBuilder.must(QueryBuilders.rangeQuery( "shopTime" ).gte( shop.getShopTime() ));
        }
        if(shop.getShopTime()!=null){
            boolQueryBuilder.must(QueryBuilders.rangeQuery( "shopTime" ).lte( shop.getShopTime() ));
        }
        searchSourceBuilder.query(boolQueryBuilder);
        //  html格式
        searchSourceBuilder.highlighter(new HighlightBuilder()
                .field( "shopName" )
                .preTags("<span style=\"color:orange\">")
                .postTags("</span>")
        );
        searchRequest.source(searchSourceBuilder);
        try {
            //      发送请求
            SearchResponse search = restHighLevelClient.search( searchRequest, RequestOptions.DEFAULT );
            //      拿到第一个hit
            SearchHits hits = search.getHits();
            //  拿到第二个hit
            SearchHit[] hits1 = hits.getHits();
            //此时hit为数据 循环遍历
            for (SearchHit documentFields : hits1) {
                //单条数据转为String类型
                String sourceAsString = documentFields.getSourceAsString();
                //转为Java对象
                Shop shop1 = JSONObject.parseObject( sourceAsString, Shop.class );
                //设置对象ID 如果类型不同 转类型
                shop1.setShopId( Integer.valueOf( documentFields.getId() ) );
                //获取高亮集合
                Map<String, HighlightField> highlightFields = documentFields.getHighlightFields();
                if(highlightFields!=null){
                    HighlightField shopName = highlightFields.get( "shopName" );
                    if(shopName!=null){
                        String str= "";
                        Text[] fragments = shopName.fragments();
                        for (Text fragment : fragments) {
                            str+=fragment;
                        }
                        shop1.setShopName( str );
                    }
                }
                tables.add( shop1 );
            }
        } catch (IOException e) {
            throw new RuntimeException( e );
        }
        return Result.success(tables);
    }
//      批量添加
    @Override
    public void batchAddShop(List<Shop> shopList) {
        BulkRequest bulkRequest = new BulkRequest();
        shopList.forEach( shop -> {
            bulkRequest.add( new IndexRequest(INDEX_NAME)
                    .id(  shop.getShopId()+"")
                    .source(JSONObject.toJSONString( shop ),XContentType.JSON ));
        } );
        try {
            restHighLevelClient.bulk( bulkRequest,RequestOptions.DEFAULT );
        } catch (IOException e) {
            throw new RuntimeException( e );
        }
    }
//    霹雳删除
    @Override
    public void batchDelShop(List<Integer> idsList) {
        BulkRequest bulkRequest = new BulkRequest();
        idsList.forEach( integer -> {
            bulkRequest.add( new DeleteRequest(INDEX_NAME,integer+""));
        } );
        try {
            restHighLevelClient.bulk( bulkRequest,RequestOptions.DEFAULT );
        } catch (IOException e) {
            throw new RuntimeException( e );
        }
    }



}
