package com.bwie.seckill.controller;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Car;
import com.bwie.common.domain.Seckill;
import com.bwie.common.domain.Shop;
import com.bwie.common.domain.User;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.seckill.service.impl.SeckillServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.List;

@RestController
public class SeckillController {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private SeckillServiceImpl seckillService;

    //秒杀列表
    @GetMapping("/findSeckillList")
    public Result<List<Seckill>> findSeckillList(){
        return Result.success(
                seckillService.findSeckillList()
        );
    }
    //秒杀详细信息（回显商品数据）
    @PostMapping("/findByshopId")
    public Result findByshopId(@RequestParam Integer shopId){
        return Result.success(
                seckillService.findByshopId( shopId )
        );
    }
    //触发添加购物车操作
    @PostMapping("/updShopJoin")
    public Result updShopJoin(@RequestBody Shop shop){
        Integer upd=seckillService.updShopJoin(shop.getShopId());
        if(upd<1){
            return Result.error(500,"添置购物车失败 请重试");
        }
        Car car = new Car();
        //购物车需要的字段
        //car_id//car_name//car_num//car_price//car_sum//car_time
        car.setCarName( shop.getShopName() );
        car.setCarNum( shop.getCarNum() );
        //商品小计计算正确（2分）
        car.setCarPrice( shop.getShopPrice() );
        //转类型相乘    商品总价计算正确（2分）
        car.setCarSum( shop.getShopPrice().multiply( BigDecimal.valueOf( shop.getCarNum() ) ) );
        //根据名字查询信息
        List<Car> carByName = seckillService.findCarByName( car.getCarName() );
        //如果存在于购物车
        if(carByName.size()!=0){
            //遍历拿到对象
            for (Car car1 : carByName) {
                //拿对象的ID
                Integer carId = car1.getCarId();
                //库存量修改 ==>原库存量+1
                seckillService.updById( carId );
            }
            return Result.success("添加成功");
        }
        //如果不存在购物车 添加至购物车
        seckillService.addCar( car );
        return Result.success("添加成功");
    }
    //触发扣款
    @PostMapping("updCarStatus")
    public Result updCarStatus(@RequestBody Car car){
//        当前操作人信息
        String token = request.getHeader( "token" );
        String userKey = JwtUtils.getUserKey( token );
        String s = redisTemplate.opsForValue().get( "token:" + userKey );
        User user = JSONObject.parseObject( s, User.class );
        Integer i = seckillService.updCarStatus( car );
        if(i<1){
            return Result.error(500,"结算失败");
        }
        //本单的总金额
        BigDecimal carSum = car.getCarSum();
        //用户的余额
        BigDecimal userMoney = user.getUserMoney();
//        使用余额进行结算，如果余额不足则支付失败（2分）
        if(userMoney.compareTo( carSum )<0){
            return Result.error(500,"请先充值");
        }
//        扣相对应的钱 BIGDECIMAL类型 调用修改方法
        BigDecimal subtract = userMoney.subtract( carSum );
        seckillService.updUserMoney( user.getUserId(),subtract );

        //结账后    删除该订单记录
        seckillService.delCar( car.getCarId() );
        //数据库 商品库存-1

        return Result.success();
    }
}
